@@ -319,10 +319,9 @@ class StackedSparseArray:
         # pylint: disable=too-many-arguments
         if self.shape[2] == 0 or (self.shape[2] == 1 and name in self.score_names):
             # Add first (sparse) array of scores
-            self.data = np.array(data, dtype=[(name, data.dtype)])
+            self.data = update_structed_array_names(data, name)
             self.row = row.copy()
             self.col = col.copy()
-            # self.__n_row, self.__n_col = coo_matrix.shape
         else:
             if join_type in ["outer", "right"]:
                 assert np.max(row) <= self.shape[0], "row values have dimension larger than sparse stack"
@@ -396,6 +395,14 @@ class StackedSparseArray:
                           shape=(self.__n_row, self.__n_col))
 
 
+def update_structed_array_names(input_array: np.ndarray, name: str):
+    if len(input_array.dtype) > 1:  # if structured array
+        dtype_new = np.dtype({'names': [f"{name}_{x}" for x in input_array.dtype.names],
+                              'formats': [x[0] for x in input_array.dtype.fields.values()]})
+        return input_array.astype(dtype_new)
+    return np.array(input_array, dtype=[(name, input_array.dtype)])
+
+
 def _unpack_index(index):
     if isinstance(index, tuple):
         if len(index) == 3:
